from ..util import is_fuzzy_key
from .asset_map import ASSET_MAP, get_request_path

def get_asset_by_id_request(asset_type, asset_id=None, lang=None, **kwargs):
    path = get_request_path(asset_type, "get")
    if asset_id is not None:
        path = f"{path}/{asset_id}"
    
    custom_headers = {}
    if lang is not None and isinstance(lang, str):
        custom_headers["X-Pandora-Language"] = lang

    return {
        "path": path,
        "query_params": {},
        # list 操作用不到的内容
        "method": "get",
        "data": {},
        "custom_headers": custom_headers,
    }



from ..util import is_fuzzy_key


ASSET_MAP = {
    # key 都小写
    "repos": {
        "path": "repos",
        "desc": "仓库",
        "query_fields": [
            {"field": "withDocSize", "required": False, "default": True},
        ],
        "methods": {
            "list": "repos",
            "get": "repos",
            "create": "repos",
        },
        "example": {
            "name": "app_mongodb_log",
            "type": "EVENTS",
            "groupPathNames": "数据存储/Mongodb",
            "description": "mongodb日志存储仓库, 默认存储 15 天",
            "retention": 1296000000,
            "writeRefreshIntervalInSeconds": 10,
            "shardMaxDocs": 50000000,
            "shardMaxSizeInMB": 51200,
            "indexMaxAgeInSecond": 86400
        },
    },
    "sourcetype": {
        "path": "sourcetype",
        "desc": "来源类型",
        "methods": {
            "list": "sourcetype",
            "get": "sourcetype",
            "create": "sourcetype",
        },
        "example": {
            "name": "sourcetype_name",
            "line": {"type": "single", "regex": ""},
            "datetime": {
                "type": "auto",
                "zoneOffset": 8,
                "dateTimeFormat": "",
                "dateTimePrefix": "",
                "maxDateTimeLength": 100,
            },
            "description": "",
            "category": "custom",
            "advance": {"charset": "utf-8", "fieldDiscovery": True},
            "preset": 0,
            "extractions": [],
        }
    },
    "extractions": {
        "path": "extractions/list",
        "desc": "字段提取规则",
        "methods": {
            "list": "extractions/list",
            "get": "extractions",
            "create": "extractions",  # 不带名字
        },
        "example": {
            "name": "extraction_name",
            "description": "",
            "type": "regex",
            "config": {
                "pattern": "^(?<a>[\\s\\S]+)",
                "delimiter": ",",
                "delimiterKv": ":"
            },
            "sourcetype": "json",
            "indexed": False,
            "app": "search",
            "schema": [
                {
                    "field": "a"
                }
            ],
            "groupIds": []
        }
    },
    "targets": {
        "path": "metric/targets",
        "desc": "运维资产对象",
        "methods": {
            "list": "metric/targets",
            "get": "metric/targets",
            "create": "metric/targets",
        },
        "example": {
            "targetTypeId": "_storage_switch",
            "identities": {"name": "target_id"},
            "names": {"name": "target_name"},
            "properties": {},
            "customProperties": {}
        }
    },
    "bizsystems": {
        "path": "target/bizSystems",
        "desc": "运维资产业务系统",
        "methods": {
            "list": "target/bizSystems",
            "get": "target/bizSystems",
            "create": "target/bizSystems",
        }
    },
    "metrics": {
        "path": "metric/metrics",
        "desc": "指标",
        "methods": {
            "list": "metric/metrics",
            "get": "metric/metrics",
            "create": "metric/metrics",
        }
    },
    "users": {
        "path": "auth/users",
        "desc": "用户",
        "methods": {
            "list": "auth/users",
        }
    },
    "roles": {
        "path": "auth/roles",
        "desc": "角色",
        "methods": {
            "list": "auth/roles",
        }
    },
    "alerts": {
        "path": "alerts",
        "desc": "告警规则",
        "methods": {
            "list": "alerts",
            "get": "alerts",
        }
    },



    # KetaAgent DC 模块
    "dc/collector/config": {
        "path": "dc/collector/config",
        "desc": "KetaAgent 采集任务配置",
        "methods": {
            "list": "dc/collector/config"
        }
    },
    "dc/collector/server/config": {
        "path": "dc/collector/server/config",
        "desc": "KetaAgent 服务端采集任务配置",
        "methods": {
            "list": "dc/collector/server/config"
        }
    },
    "dc/kubernetes/collector/rule": {
        "path": "dc/kubernetes/collector/rule",
        "desc": "KetaAgent 容器采集任务",
        "methods": {
            "list": "dc/kubernetes/collector/rule"
        }
    },
    "dc/agent": {
        "path": "dc/agent/list",
        "desc": "KetaAgent Agent实例",
        "methods": {
            "list": "dc/agent/list"
        }
    },
    "dc/tags": {
        "path": "dc/tags",
        "desc": "KetaAgent Agent标签",
        "methods": {
            "list": "dc/tags"
        }
    },
    "dc/collect/types": {
        "path": "dc/collect/types",
        "desc": "KetaAgent 采集模板",
        "methods": {
            "list": "dc/collect/types"
        }
    },
    "dc/transformer/types": {
        "path": "dc/transformer/types",
        "desc": "KetaAgent 解析模板",
        "methods": {
            "list": "dc/transformer/types"
        }
    },
    "dc/package": {
        "path": "dc/package/list",
        "desc": "KetaAgent 安装包",
        "methods": {
            "list": "dc/package/list"
        }
    },
}


def get_request_path(asset_type, method):
    path = asset_type
    
    key = is_fuzzy_key(asset_type, value_map=ASSET_MAP)
    if "methods" in ASSET_MAP[key] and method in ASSET_MAP[key]["methods"]:
        path = ASSET_MAP.get(key)["methods"][method]
    else:
        path = ASSET_MAP.get(key)["path"]
    return path
        
    



import json 
import argcomplete

from mando import command, main
from datetime import datetime

from ketacli.sdk.base.client import *
from ketacli.sdk.base.search import search_spl
from ketacli.sdk.request.list import list_assets_request
from ketacli.sdk.request.create import create_asset_request
from ketacli.sdk.request.get import get_asset_by_id_request

from ketacli.sdk.output.output import list_output, describe_output, get_asset_output
from ketacli.sdk.output.output import search_result_output
from ketacli.sdk.output.format import format_table


@command
def login(name="keta", endpoint="http://localhost:9000", token=""):
    """Login to ketadb, cache authentication info to ~/.keta/config.yaml

    :param repository: Repository to push to.
    :param -n, --name: The login account name. Defaults to "keta".
    :param -e, --endpoint: The ketadb endpoint. Defaults to "http://localhost:9000".
    :param -t, --token: Your keta api token, create from ketadb webui. Defaults to "".
    """
    do_login(name=name, endpoint=endpoint, token=token)


@command
def logout():
    """Logout from ketadb, clear authentication info"""
    do_logout()


@command
def list(asset_type, groupId=-1, order="desc", pageNo=1, pageSize=10, prefix="", sort="updateTime", fields="", format=None, raw=False,
         lang=None):
    """List asset (such as repo,sourcetype,metric...) from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -l, --pageSize: Limit the page size.
    :param --pageNo: Limit the page number.
    :param --prefix: Fuzzy query filter.
    :param --sort: The field used to order by
    :param --order: The sort order, desc|asc
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --groupId: The resource group id.
    :param --raw: Prettify the time field or output the raw timestamp, if specified, output the raw format
    :param --lang: Choose the language preference of return value
    """
    req = list_assets_request(
        asset_type, groupId, order, pageNo, pageSize, prefix, sort, lang)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    output_fields = []
    if len(fields.strip()) > 0:
        output_fields = fields.strip().split(",")
    table = list_output(asset_type, output_fields=output_fields, resp=resp)
    if table is None:
        print(f"we cannot find any {asset_type}")
    else:
        print(format_table(table, format, not raw))


@command
def get(asset_type, asset_id, fields="", format=None, lang=None):
    """Get asset detail info from ketadb

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param asset_id: The unique id of asset. (such as id or name...)
    :param --fields: The fields to display. Separate by comman, such as "id,name,type"
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --lang: Choose the language preference of return value
    """
    req = get_asset_by_id_request(
        asset_type=asset_type, asset_id=asset_id, lang=lang)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    if format == "json":
        print(json.dumps(resp, indent=2, ensure_ascii=False))
        return
    
    output_fields = []
    if len(fields.strip()) > 0:
        output_fields = fields.strip().split(",")
    table = get_asset_output(output_fields=output_fields, resp=resp)
    table.align = "l"
    if table is None:
        print(f"we cannot find any {asset_type}")
    else:
        print(format_table(table, format))


@command
def describe(asset_type, format=None):
    """Describe the schema of asset type

    :param asset_type: The asset type such as repo, sourcetype, metirc, targets ...
    :param -f, --format: The output format, text|json|csv|html|latex
    """

    req = list_assets_request(asset_type)
    resp = request_get(req["path"], req["query_params"],
                       req["custom_headers"]).json()
    table = describe_output(asset_type, resp=resp)
    if table is None:
        print(f"we cannot find any {asset_type}")
    else:
        print(format_table(table, format))


@command
def search(spl, start=None, end=None, limit=100, format=None, raw=False):
    """Search spl from ketadb

    :param spl: The spl query 
    :param --start: The start time. Time format "2024-01-02 10:10:10"
    :param --end: The start time. Time format "2024-01-02 10:10:10"
    :param -l, --limit: The limit size of query result
    :param -f, --format: The output format, text|json|csv|html|latex
    :param --raw: Prettify the time field or output the raw timestamp, if specified, output the raw format
    """
    if start != None:
        start = datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
    if end != None:
        end = datetime.strptime(start, '%Y-%m-%d %H:%M:%S')
    resp = search_spl(spl=spl, start=start, end=end, limit=limit)
    table = search_result_output(resp)
    if table is None:
        print(f"we cannot find any data")
    else:
        print(format_table(table, format, not raw))


@command
def insert(repo="default", data=None, file=None):
    """Upload data to specified repo

    :param --repo: The target repo
    :param --data: The json string data [{"raw":"this is text", "host": "host-1"}]
    :param --file: Upload json text from file path.
    """
    if repo is None:
        print(f"Please specify target repo with --repo")
        return
    if data is None and file is None:
        print(f"Please use --data or --file to specify data to upload")
        return

    if file is not None:
        f = open(file)
        data = f.read()

    query_params = {
        "repo": repo,
    }
    resp = request_post("data", json.loads(data), query_params).json()
    print(resp)


@command
def create(asset_type, name=None, data=None, file=None):
    """Create asset 

    :param asset_type: The target asset type, such as repo, sourcetype ...
    :param -n --name: The target asset name
    :param --data: The json string data {...}
    :param --file: Upload json text from file path.
    """
    if data is None and file is None:
        print(f"Please use --data or --file to specify parameters to create asset")
        return
    if file is not None:
        f = open(file)
        data = f.read()
    req = create_asset_request(asset_type, name, data)
    resp = request(req["method"], req["path"], data=data).json()
    print(resp)


if __name__ == "__main__":
    argcomplete.autocomplete(main.parser)
    main()



from ..util import is_fuzzy_key
from .asset_map import ASSET_MAP

def list_assets_request(asset_name, groupId=-1, order="desc", pageNo=1,
                pageSize=10, prefix="", sort="updateTime", lang=None, **kwargs):
    path = asset_name
    query_fields = []
    key = is_fuzzy_key(asset_name, value_map=ASSET_MAP)
    if key != None:
        path = ASSET_MAP.get(key)["path"]
        if "query_fields" in ASSET_MAP.get(key):
            query_fields = ASSET_MAP.get(key)["query_fields"]
    query_params = {
        "groupId": groupId,
        "order": order,
        "pageNo": pageNo,
        "pageSize": pageSize,
        "prefix": prefix,
        "sort": sort,
    }
    for finfo in query_fields:
        f = finfo["field"]
        dft = finfo["default"]
        required = finfo["required"]
        if f in kwargs:
            query_params[f] = kwargs[f]
        elif required:
            query_params[f] = dft

    custom_headers = {}
    if lang is not None and isinstance(lang, str):
        custom_headers["X-Pandora-Language"] = lang
        
    return {
        "path": path,
        "query_params": query_params,
        # list 操作用不到的内容
        "method": "get",
        "data": {},
        "custom_headers": custom_headers,
    }





def is_key(key, value_map={}):
    """精确匹配是否存在key

    Args:
        key (str): 字符串key
        value_map (dict, optional): 是否存在该key的字典. Defaults to {}.

    Returns:
        _type_: 最终的key，如果不存在则为None
    """
    newkey = key.lower()
    if newkey in value_map:
        return newkey
    return None

def is_fuzzy_key(key, value_map={}):
    """
    检查key或者key的复数形式在map中，并返回最终map中的key
    """
    newkey = key.lower() 
    if newkey in value_map:
        return newkey
    if newkey.endswith('s'):
        newkey = newkey[:-1]
    else:
        newkey = newkey + 's'
    if newkey in value_map:
        return newkey
    return None


import json

from datetime import datetime
from .format import make_records_to_table, make_table, make_table, prettify_value
from ..util import is_fuzzy_key

RESP_OUTPUT_KEY = {
    "bizsystems": "items",
    "target": "items",
    "targettype": "items",
}


def find_list_field(resp={}):
    """找寻返回结果中的list类型字段，用来查找分页返回结果中应该处理字段的函数

    Args:
        resp (dict, optional): 分页返回结果. Defaults to {}.

    Returns:
        str | None: 第一个list类型字段的key，如果没有则返回None
    """
    for k in resp:
        if isinstance(resp[k], list):
            return k
    return None


def find_result_field(asset_type, resp={}):
    ""
    # 检查白名单有无这个字段的定义，如果有，则直接使用
    real_asset_key = is_fuzzy_key(asset_type, value_map=RESP_OUTPUT_KEY)
    if real_asset_key is not None:
        return RESP_OUTPUT_KEY.get(real_asset_key)

    key = None
    # 优先查看返回结果里是否有类似命名的字段
    key = is_fuzzy_key(asset_type, value_map=resp)

    # 如果最终没有找到类似的字段，则找一个list字段
    if key is None:
        key = find_list_field(resp)

    # 如果返回结果为空
    if key is None or not isinstance(resp[key], list) or len(resp[key]) <= 0:
        return None

    return key


def list_output(asset_type, output_fields=[], resp={}):
    total = resp.get("total")
    if total is None:
        total = 0
    print(f"we have {total} {asset_type} in total")

    result_field = find_result_field(asset_type, resp)
    if result_field is None:
        return None

    table = make_records_to_table(output_fields, resp[result_field])
    return table


def search_result_output(result={}):
    header = []
    for f in result["fields"]:
        header.append(f["name"])
    rows = result["rows"]
    return make_table(header, rows)


def get_asset_output(resp={}, output_fields=[]):
    header = ["field", "value"]
    fields = []
    filter_field = len(output_fields) > 0
    output_fields = set(output_fields)
    for k in resp:
        if (not filter_field) or (k in output_fields):
            fields.append([k, prettify_value(k, resp[k])])
    table = make_table(header, fields)
    return table


def describe_output(asset_type, resp={}):
    """通过result字段推断这个资源返回字段的类型

    Args:
        asset_type (str): 资源的类型，如repo、dashboard等
        resp (dict, optional): 请求的返回体. Defaults to {}.

    Returns:
        PrettyTable | None: 返回格式化好的表格，或者如果没有result字段则返回None
    """
    total = resp.get("total")
    if total is None or total <= 0:
        return None

    result_field = find_result_field(asset_type, resp)
    if result_field is None:
        return None

    header = ["fields", "type"]
    fields = []
    for k in resp[result_field][0]:
        fields.append((k, type(resp[result_field][0][k])))
    table = make_table(header, fields)
    return table



from ..util import is_fuzzy_key
from .asset_map import ASSET_MAP
from .asset_map import get_request_path

def create_asset_request(asset_type, name=None, data=None, **kwargs):
    path = path = get_request_path(asset_type, "get")
    if name is not None:
        path = f"{path}/{name}"

    return {
        "path": path,
        "query_params": {},
        # list 操作用不到的内容
        "method": "post",
        "data": data,
        "custom_headers": {},
    }



import requests
import yaml
import os
import json

ROOT_PATH = "api/v1"

ENV_ENDPOINT = 'KETA_SERVICE_ENDPOINT'
ENV_TOKEN = 'KETA_SERVICE_TOKEN'

AUTH_FILE_PATH = '~/.keta/config.yaml'


def do_login(name, endpoint, token):
    # Attempt to access the endpoint
    response = requests.get(
        endpoint, headers={"Authorization": f"Bearer {token}"})
    if response.status_code == 200:
        # Ensure the configuration directory exists
        config_dir = os.path.expanduser('~/.keta')
        os.makedirs(config_dir, exist_ok=True)
        config_path = os.path.join(config_dir, 'config.yaml')
        # Save login information to the configuration file
        with open(config_path, 'w') as file:
            yaml.dump({'name': name, 'endpoint': endpoint, 'token': token}, file)
        print("Login successful, information saved.")
    else:
        print("Login failed, please check your endpoint and token.")


def do_logout():
    # Delete the configuration file
    config_path = os.path.expanduser(AUTH_FILE_PATH)
    if os.path.exists(config_path):
        os.remove(config_path)
        print("Logout successful, login information deleted.")
    else:
        print("You are not logged in or already logged out.")


def get_auth():
    # 尝试从环境变量获取endpoint和token
    endpoint = os.getenv(ENV_ENDPOINT)
    token = os.getenv(ENV_TOKEN)

    # 如果环境变量为空，尝试从配置文件读取
    if not endpoint or not token:
        config_path = os.path.expanduser(AUTH_FILE_PATH)
        if os.path.exists(config_path):
            with open(config_path, 'r') as file:
                config = yaml.safe_load(file)
                endpoint = config.get('endpoint')
                token = config.get('token')
        else:
            print("No authentication information available.")
            return {}

    # 返回认证信息
    return {'endpoint': endpoint, 'token': token}


def request(method, path, data={}, query_params={}, custom_headers={}):
    # 从getAuth获取认证信息
    auth_info = get_auth()
    if (auth_info is None or len(auth_info) == 0):
        print("please login first")
        exit(1)

    endpoint = auth_info.get('endpoint')
    token = auth_info.get('token')

    # 确保endpoint已提供
    if not endpoint or not token:
        print("Endpoint or token is not provided.")
        return None

    # 拼接完整的URL
    url = ""
    if ROOT_PATH in path:
        url = f"{endpoint.rstrip('/')}/{path.lstrip('/')}"
    else:
        url = f"{endpoint.rstrip('/')}/{ROOT_PATH}/{path.lstrip('/')}"

    # 准备请求头，加入认证Token
    headers = {'Authorization': f"Bearer {token}",
               'Content-Type': "application/json"}
    # 加入任何自定义的请求头
    headers.update(custom_headers)

    if isinstance(data, str):
        response = requests.request(
            method, data=data, url=url, params=query_params, headers=headers)
    else:
        response = requests.request(
            method, json=data, url=url, params=query_params, headers=headers)

    if response.status_code >= 400 and response.status_code < 500:
        raise Exception("Bad request", response.status_code,
                        response.json())
    if response.status_code >= 500 and response.status_code < 600:
        raise Exception("Server error", response.status_code,
                        response.json())
    # 返回响应
    return response


def request_get(path, query_params={}, custom_headers={}):
    return request('get', path, query_params=query_params, custom_headers=custom_headers)


def request_post(path, data={}, query_params={}, custom_headers={}):
    return request('post', path, data=data, query_params=query_params, custom_headers=custom_headers)


def request_put(path, data={}, query_params={}, custom_headers={}):
    return request('put', path, data=data, query_params=query_params, custom_headers=custom_headers)


def request_delete(path, data={}, query_params={}, custom_headers={}):
    return request('delete', path, query_params=query_params, custom_headers=custom_headers)



from setuptools import setup, find_packages

setup(
    name='ketacli',
    version='0.8',
    packages=find_packages(),
    license='MIT',
    description='KetaDB Client',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    author='lvheyang',
    author_email='cuiwenzheng@ymail.com',
    url='https://xishuhq.com',
    install_requires=[
        "requests",
        "prettytable",
        "pyyaml",
        "mando",
    ],
    entry_points={
        'console_scripts': [
            'ketacli=ketacli.ketacli:main',
        ],
    },
)
